@@ -3,21 +3,21 @@
     Copyright (C) 1998  Itai Tzur <itzur@actcom.co.il>
     Copyright (C) 1999  Sean 'Shaleh' Perry <shaleh@debian.org>
     Copyright (C) 2004  Pascal Hakim <pasc@redellipse.net>
- 
+
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
- 
+
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
- 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- 
+
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+
     The GNU General Public License can also be found in the file
     `COPYING' that comes with the Anacron source distribution.
 */
@@ -81,7 +81,7 @@ conv2int(const char *s)
 }
 
 static char *
-read_tab_line ()
+read_tab_line (void)
 /* Read one line and return a pointer to it.
 Return NULL if no more lines.
  */
@@ -110,15 +110,15 @@ Return NULL if no more lines.
 
 static int
 job_arg_num(const char *ident)
-/* Return the command-line-argument number refering to this job-identifier.
+/* Return the command-line-argument number referring to this job-identifier.
  * If it isn't specified, return -1.
  */
 {
     int i, r;
 
-    for (i = 0; i < nargs; i++)
+    for (i = 0; i < job_nargs; i++)
     {
-	r = fnmatch(args[i], ident, 0);
+	r = fnmatch(job_args[i], ident, 0);
 	if (r == 0) return i;
 	if (r != FNM_NOMATCH) die("fnmatch() error");
     }
@@ -194,9 +194,8 @@ register_period_job(const char *periods, const char *delays,
 {
     int delay;
     job_rec *jr;
-    int period_len, ident_len, command_len;
+    int ident_len, command_len;
 
-    period_len = strlen(periods);
     ident_len = strlen(ident);
     command_len = strlen(command);
     jobs_read++;
@@ -209,11 +208,11 @@ register_period_job(const char *periods, const char *delays,
     }
 
     jr = obstack_alloc(&tab_o, sizeof(job_rec));
-    if (!strncmp ("@monthly", periods, 7)) {
+    if (!strncmp ("@monthly", periods, 8)) {
 		jr->named_period = 1;
-    } else if (!strncmp("@yearly", periods, 7) || !strncmp("@annualy", periods, 8)) {
+    } else if (!strncmp("@yearly", periods, 7) || !strncmp("@annually", periods, 9) || !strncmp(/* backwards compat misspelling */"@annualy", periods, 8)) {
 		jr->named_period = 2;
-	} else if (!strncmp ("@daily", periods, 7)) {
+	} else if (!strncmp ("@daily", periods, 6)) {
 		jr->named_period = 3;
 	} else if (!strncmp ("@weekly", periods, 7)) {
 		jr->named_period = 4;
@@ -252,6 +251,7 @@ parse_tab_line(char *line)
     char *command;
     char *from;
     char *to;
+    char *pref_hour;
 
     /* an empty line? */
     r = match_rx("^[ \t]*($|#)", line, 0);
@@ -271,13 +271,20 @@ parse_tab_line(char *line)
         if (strncmp(env_var, "START_HOURS_RANGE", 17) == 0)
         {
             r = match_rx("^([[:digit:]]+)-([[:digit:]]+)$", value, 2, &from, &to);
-            if ((r == -1) || (from == NULL) || (to == NULL)) goto reg_invalid;
+            if (r == -1) goto reg_err;
+            if (r == 0) goto reg_invalid;
             range_start = atoi(from);
             range_stop = atoi(to);
+            if (range_stop < range_start) {
+                range_start = 0; range_stop = 0;
+                goto reg_invalid;
+            }
             Debug(("Jobs will start in the %02d:00-%02d:00 range.", range_start, range_stop));
         }
         if (strncmp(env_var, "RANDOM_DELAY", 12) == 0) {
             r = match_rx("^([[:digit:]]+)$", value, 0);
+            if (r == -1) goto reg_err;
+            if (r == 0) goto reg_invalid;
             if (r != -1) {
                 int i = random();
                 double x = 0;
@@ -285,7 +292,16 @@ parse_tab_line(char *line)
                 random_number = (int)x;
                 Debug(("Randomized delay set: %d", random_number));
             }
-        else goto reg_invalid;
+        }
+        if (strncmp(env_var, "PREFERRED_HOUR", 14) == 0) {
+            r = match_rx("^([[:digit:]]+)$", value, 1, &pref_hour);
+            if ((r != -1) || (pref_hour != NULL)) {
+                preferred_hour = atoi(pref_hour);
+                if ((preferred_hour < 0) || (preferred_hour > 24)) {
+                    preferred_hour = -1;
+                    goto reg_invalid;
+                }
+            }
         }
 	register_env(env_var, value);
 	return;
@@ -366,7 +382,7 @@ execution_order(const job_rec **job1, const job_rec **job2)
 }
 
 void
-arrange_jobs()
+arrange_jobs(void)
 /* Make an array of pointers to jobs that are going to be executed,
  * and arrange them in the order of execution.
  * Also lock these jobs.
