@@ -24,9 +24,35 @@
  * to add clustering support.
  */
 
+#include "config.h"
+
 #define	MAIN_PROGRAM
 
-#include <cron.h>
+#include <errno.h>
+#include <langinfo.h>
+#include <locale.h>
+#include <pwd.h>
+#include <signal.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <sys/wait.h>
+#include <unistd.h>
+#include <sys/time.h>
+
+#ifdef WITH_INOTIFY
+# include <sys/inotify.h>
+#endif
+
+#ifdef HAVE_SYS_FCNTL_H
+# include <sys/fcntl.h>
+#endif
+
+#include "cronie_common.h"
+#include "funcs.h"
+#include "globals.h"
+#include "pathnames.h"
 
 #if defined WITH_INOTIFY
 int inotify_enabled;
@@ -36,16 +62,16 @@ int inotify_enabled;
 
 enum timejump { negative, small, medium, large };
 
-static void usage(void),
+static void usage(void) ATTRIBUTE_NORETURN,
 run_reboot_jobs(cron_db *),
 find_jobs(int, cron_db *, int, int, long),
 set_time(int),
 cron_sleep(int, cron_db *),
 sigchld_handler(int),
 sighup_handler(int),
-sigchld_reaper(void), quit(int), parse_args(int c, char *v[]);
+sigchld_reaper(void), sigintterm_handler(int), parse_args(int c, char *v[]);
 
-static volatile sig_atomic_t got_sighup, got_sigchld;
+static volatile sig_atomic_t got_sighup, got_sigchld, got_sigintterm;
 static int timeRunning, virtualTime, clockTime;
 static long GMToff;
 static int DisableInotify;
@@ -59,16 +85,30 @@ static int DisableInotify;
  * clustering enabled.
  */
 
-# define NUM_WATCHES 3
+# if defined ENABLE_SYSCRONTAB
+# 	define NUM_WATCHES 3
 
 int wd[NUM_WATCHES];
 const char *watchpaths[NUM_WATCHES] = {SPOOL_DIR, SYS_CROND_DIR, SYSCRONTAB};
+# else
+#	define NUM_WATCHES 2
+int wd[NUM_WATCHES];
+const char *watchpaths[NUM_WATCHES] = {SPOOL_DIR, SYS_CROND_DIR};
+# endif
 
-void set_cron_unwatched(int fd) {
-	int i;
+static void reset_watches(void) {
+	size_t i;
 
 	for (i = 0; i < sizeof (wd) / sizeof (wd[0]); ++i) {
-		if (wd[i] < 0) {
+		wd[i] = -2;
+	}
+}
+
+void set_cron_unwatched(int fd) {
+	size_t i;
+
+	for (i = 0; i < sizeof (wd) / sizeof (wd[0]); ++i) {
+		if (wd[i] > 0) {
 			inotify_rm_watch(fd, wd[i]);
 			wd[i] = -1;
 		}
@@ -77,7 +117,7 @@ void set_cron_unwatched(int fd) {
 
 void set_cron_watched(int fd) {
 	pid_t pid = getpid();
-	int i;
+	size_t i;
 
 	if (fd < 0) {
 		inotify_enabled = 0;
@@ -87,22 +127,21 @@ void set_cron_watched(int fd) {
 	for (i = 0; i < sizeof (wd) / sizeof (wd[0]); ++i) {
 		int w;
 
-		if (open(watchpaths[i], O_RDONLY | O_NONBLOCK, 0) != -1) {
-			w = inotify_add_watch(fd, watchpaths[i],
-				IN_CREATE | IN_CLOSE_WRITE | IN_ATTRIB | IN_MODIFY | IN_MOVED_TO |
-				IN_MOVED_FROM | IN_MOVE_SELF | IN_DELETE | IN_DELETE_SELF);
-			if (w < 0) {
-				if (wd[i] != -1) {
-					log_it("CRON", pid, "This directory or file can't be watched",
-						watchpaths[i], errno);
-					log_it("CRON", pid, "INFO", "running without inotify support", 0);
-				}
-				inotify_enabled = 0;
-				set_cron_unwatched(fd);
-				return;
+		w = inotify_add_watch(fd, watchpaths[i],
+			IN_CREATE | IN_CLOSE_WRITE | IN_ATTRIB | IN_MODIFY | IN_MOVED_TO |
+			IN_MOVED_FROM | IN_MOVE_SELF | IN_DELETE | IN_DELETE_SELF);
+		if (w < 0 && errno != ENOENT) {
+			if (wd[i] != -1) {
+				log_it("CRON", pid, "This directory or file can't be watched",
+					watchpaths[i], errno);
+				log_it("CRON", pid, "INFO", "running without inotify support",
+					0);
 			}
-			wd[i] = w;
+			inotify_enabled = 0;
+			set_cron_unwatched(fd);
+			return;
 		}
+		wd[i] = w;
 	}
 
 	if (!inotify_enabled) {
@@ -120,6 +159,7 @@ static void handle_signals(cron_db * database) {
 		/* watches must be reinstated on reload */
 		if (inotify_enabled && (EnableClustering != 1)) {
 			set_cron_unwatched(database->ifd);
+			reset_watches();
 			inotify_enabled = 0;
 		}
 #endif
@@ -136,18 +176,23 @@ static void handle_signals(cron_db * database) {
 static void usage(void) {
 	const char **dflags;
 
-	fprintf(stderr, "usage:  %s [-h] print this message \n \
-		[-i] deamon runs without inotify support \n \
-		[-m <mail command>] off or specify prefered client for sending mails \n \
-		[-n] run in foreground \n \
-		[-p] permit any crontab \n \
-		[-c] enable clustering support \n \
-		[-s] log into syslog instead of sending mails \n \
-		[-x [",
-		ProgramName);
+	fprintf(stderr, "Usage:\n");
+	fprintf(stderr, " %s [options]\n", ProgramName);
+	fprintf(stderr, "\n");
+	fprintf(stderr, "Options:\n");
+	fprintf(stderr, " -h         print this message \n");
+	fprintf(stderr, " -i         deamon runs without inotify support\n");
+	fprintf(stderr, " -m <comm>  off, or specify preferred client for sending mails\n");
+	fprintf(stderr, " -n         run in foreground\n");
+	fprintf(stderr, " -p         permit any crontab\n");
+	fprintf(stderr, " -P         use PATH=\"%s\"\n", _PATH_DEFPATH);
+	fprintf(stderr, " -c         enable clustering support\n");
+	fprintf(stderr, " -s         log into syslog instead of sending mails\n");
+	fprintf(stderr, " -x <flag>  print debug information\n");
+	fprintf(stderr, "\n");
+	fprintf(stderr, "Debugging flags are: ");
 	for (dflags = DebugFlagNames; *dflags; dflags++)
-		fprintf(stderr, "%s%s", *dflags, dflags[1] ? "," : "]");
-	fprintf(stderr, "] print debug information\n");
+		fprintf(stderr, "%s%s", *dflags, dflags[1] ? "," : "\n");
 	exit(ERROR_EXIT);
 }
 
@@ -158,11 +203,17 @@ int main(int argc, char *argv[]) {
 	char *cs;
 	pid_t pid = getpid();
 	long oldGMToff;
-#if defined WITH_INOTIFY
-	int i;
-#endif
+	struct timeval tv;
+	struct timezone tz;
+	char buf[256];
+
+	if ((ProgramName=strrchr(argv[0], '/')) == NULL) {
+		ProgramName = argv[0];
+	}
+	else {
+		++ProgramName;
+	}
 
-	ProgramName = argv[0];
 	MailCmd[0] = '\0';
 	cron_default_mail_charset[0] = '\0';
 
@@ -175,9 +226,10 @@ int main(int argc, char *argv[]) {
 
 	SyslogOutput = 0;
 	NoFork = 0;
+	ChangePath = 1;
 	parse_args(argc, argv);
 
-	bzero((char *) &sact, sizeof sact);
+	memset((char *) &sact, 0, sizeof sact);
 	sigemptyset(&sact.sa_mask);
 	sact.sa_flags = 0;
 #ifdef SA_RESTART
@@ -187,7 +239,7 @@ int main(int argc, char *argv[]) {
 	(void) sigaction(SIGCHLD, &sact, NULL);
 	sact.sa_handler = sighup_handler;
 	(void) sigaction(SIGHUP, &sact, NULL);
-	sact.sa_handler = quit;
+	sact.sa_handler = sigintterm_handler;
 	(void) sigaction(SIGINT, &sact, NULL);
 	(void) sigaction(SIGTERM, &sact, NULL);
 
@@ -195,9 +247,12 @@ int main(int argc, char *argv[]) {
 	set_cron_uid();
 	check_spool_dir();
 
-	if (putenv("PATH=" _PATH_DEFPATH) < 0) {
-		log_it("CRON", pid, "DEATH", "can't putenv PATH", errno);
-		exit(1);
+	if (ChangePath) {
+		if (setenv("PATH", _PATH_DEFPATH, 1) < 0) {
+			log_it("CRON", pid, "DEATH", "can't setenv PATH",
+				errno);
+			exit(1);
+		}
 	}
 
 	/* Get the default locale character set for the mail 
@@ -241,12 +296,21 @@ int main(int argc, char *argv[]) {
 		}
 	}
 
-	if (access("/usr/sbin/sendmail", X_OK) != 0) {
+	if (!SyslogOutput && MailCmd[0] == '\0' && access("/usr/sbin/sendmail", X_OK) != 0) {
 		SyslogOutput=1;
-		log_it("CRON", pid, "INFO","Syslog will be used instead of sendmail.", errno);
+		log_it("CRON", pid, "INFO","Syslog will be used instead of sendmail.", 0);
 	}
 
 	pid = getpid();
+
+	/* obtain a random scaling factor for RANDOM_DELAY */
+	if (gettimeofday(&tv, &tz) != 0)
+		tv.tv_usec = 0;
+	srandom(pid + tv.tv_usec);
+	RandomScale = random() / (double)RAND_MAX;
+	snprintf(buf, sizeof(buf), "RANDOM_DELAY will be scaled with factor %d%% if used.", (int)(RandomScale*100));
+	log_it("CRON", pid, "INFO", buf, 0);
+
 	acquire_daemonlock(0);
 	database.head = NULL;
 	database.tail = NULL;
@@ -261,13 +325,7 @@ int main(int argc, char *argv[]) {
 			"", 0);
 	}
 	else {
-		for (i = 0; i < sizeof (wd) / sizeof (wd[0]); ++i) {
-			/* initialize to negative number other than -1
-			 * so an eventual error is reported for the first time
-			 */
-			wd[i] = -2;
-		}
-
+		reset_watches();
 		database.ifd = fd = inotify_init();
 		fcntl(fd, F_SETFD, FD_CLOEXEC);
 		if (fd < 0)
@@ -291,7 +349,7 @@ int main(int argc, char *argv[]) {
 	 * timeRunning: is the time we last awakened.
 	 * clockTime: is the time when set_time was last called.
 	 */
-	while (TRUE) {
+	while (!got_sigintterm) {
 		int timeDiff;
 		enum timejump wakeupKind;
 
@@ -299,7 +357,9 @@ int main(int argc, char *argv[]) {
 		do {
 			cron_sleep(timeRunning + 1, &database);
 			set_time(FALSE);
-		} while (clockTime == timeRunning);
+		} while (!got_sigintterm && clockTime == timeRunning);
+		if (got_sigintterm)
+			break;
 		timeRunning = clockTime;
 
 		/*
@@ -345,7 +405,7 @@ int main(int argc, char *argv[]) {
 				 * minute until caught up.
 				 */
 				Debug(DSCH, ("[%ld], normal case %d minutes to go\n",
-						(long) pid, timeDiff))
+						(long) pid, timeDiff));
 				do {
 					if (job_runqueue())
 						sleep(10);
@@ -370,7 +430,7 @@ int main(int argc, char *argv[]) {
 				 * housekeeping.
 				 */
 				Debug(DSCH, ("[%ld], DST begins %d minutes to go\n",
-						(long) pid, timeDiff))
+						(long) pid, timeDiff));
 				/* run wildcard jobs for current minute */
 				find_jobs(timeRunning, &database, TRUE, FALSE, GMToff);
 
@@ -397,7 +457,7 @@ int main(int argc, char *argv[]) {
 				 * change until we are caught up.
 				 */
 				Debug(DSCH, ("[%ld], DST ends %d minutes to go\n",
-						(long) pid, timeDiff))
+						(long) pid, timeDiff));
 				find_jobs(timeRunning, &database, TRUE, FALSE, GMToff);
 				break;
 			default:
@@ -405,7 +465,7 @@ int main(int argc, char *argv[]) {
 				 * other: time has changed a *lot*,
 				 * jump virtual time, and run everything
 				 */
-				Debug(DSCH, ("[%ld], clock jumped\n", (long) pid))
+				Debug(DSCH, ("[%ld], clock jumped\n", (long) pid));
 				virtualTime = timeRunning;
 				oldGMToff = GMToff;
 				find_jobs(timeRunning, &database, TRUE, TRUE, GMToff);
@@ -425,6 +485,10 @@ int main(int argc, char *argv[]) {
 	if (fd >= 0 && close(fd) < 0)
 		log_it("CRON", pid, "INFO", "Inotify close failed", errno);
 #endif
+
+	(void) unlink(_PATH_CRON_PID);
+
+	return 0;
 }
 
 static void run_reboot_jobs(cron_db * db) {
@@ -457,8 +521,6 @@ static void run_reboot_jobs(cron_db * db) {
 
 static void find_jobs(int vtime, cron_db * db, int doWild, int doNonWild, long vGMToff) {
 	char *orig_tz, *job_tz;
-	time_t virtualSecond = vtime * SECONDS_PER_MINUTE;
-	time_t virtualGMTSecond = virtualSecond - vGMToff;
 	struct tm *tm;
 	int minute, hour, dom, month, dow;
 	user *u;
@@ -491,11 +553,7 @@ static void find_jobs(int vtime, cron_db * db, int doWild, int doNonWild, long v
 	} while (0)
 
 	orig_tz = getenv("TZ");
-	maketime(NULL, orig_tz);
 
-	Debug(DSCH, ("[%ld] tick(%d,%d,%d,%d,%d) %s %s\n",
-			(long) getpid(), minute, hour, dom, month, dow,
-			doWild ? " " : "No wildcard", doNonWild ? " " : "Wildcard only"))
 		/* the dom/dow situation is odd.  '* * 1,15 * Sun' will run on the
 		 * first and fifteenth AND every Sunday;  '* * * * Sun' will run *only*
 		 * on Sundays;  '* * 1,15 * *' will run *only* the 1st and 15th.  this
@@ -506,10 +564,12 @@ static void find_jobs(int vtime, cron_db * db, int doWild, int doNonWild, long v
 		for (e = u->crontab; e != NULL; e = e->next) {
 			Debug(DSCH | DEXT, ("user [%s:%ld:%ld:...] cmd=\"%s\"\n",
 					e->pwd->pw_name, (long) e->pwd->pw_uid,
-					(long) e->pwd->pw_gid, e->cmd))
+					(long) e->pwd->pw_gid, e->cmd));
 				uname = e->pwd->pw_name;
 			/* check if user exists in time of job is being run f.e. ldap */
 			if (getpwnam(uname) != NULL) {
+				time_t virtualSecond = (vtime - e->delay) * SECONDS_PER_MINUTE;
+				time_t virtualGMTSecond = virtualSecond - vGMToff;
 				job_tz = env_get("CRON_TZ", e->envp);
 				maketime(job_tz, orig_tz);
 				/* here we test whether time is NOW */
@@ -533,6 +593,10 @@ static void find_jobs(int vtime, cron_db * db, int doWild, int doNonWild, long v
 						job_add(e, u);	/*will add job, if it isn't in queue already for NOW. */
 				}
 			}
+			else {
+				log_it(uname, getpid(), "ERROR", "getpwnam() failed",errno);
+				Debug(DSCH | DEXT, ("%s:%d pid=%d time=%ld getpwnam(%s) failed errno=%d error=%s\n",__FILE__,__LINE__,getpid(),time(NULL),uname,errno,strerror(errno)));
+			}
 		}
 	}
 	if (orig_tz != NULL)
@@ -557,7 +621,7 @@ static void set_time(int initialize) {
 	if (initialize || tm.tm_isdst != isdst) {
 		isdst = tm.tm_isdst;
 		GMToff = get_gmtoff(&StartTime, &tm);
-		Debug(DSCH, ("[%ld] GMToff=%ld\n", (long) getpid(), (long) GMToff))
+		Debug(DSCH, ("[%ld] GMToff=%ld\n", (long) getpid(), (long) GMToff));
 	}
 	clockTime = (StartTime + GMToff) / (time_t) SECONDS_PER_MINUTE;
 }
@@ -573,11 +637,14 @@ static void cron_sleep(int target, cron_db * db) {
 	seconds_to_wait = (int) (target * SECONDS_PER_MINUTE - t1) + 1;
 	Debug(DSCH, ("[%ld] Target time=%ld, sec-to-wait=%d\n",
 			(long) getpid(), (long) target * SECONDS_PER_MINUTE,
-			seconds_to_wait))
+			seconds_to_wait));
 
-		while (seconds_to_wait > 0 && seconds_to_wait < 65) {
+	while (seconds_to_wait > 0 && seconds_to_wait < 65) {
 		sleep((unsigned int) seconds_to_wait);
 
+		if (got_sigintterm)
+			return;
+
 		/*
 		 * Check to see if we were interrupted by a signal.
 		 * If so, service the signal(s) then continue sleeping
@@ -599,9 +666,8 @@ static void sigchld_handler(int x) {
 	got_sigchld = 1;
 }
 
-static void quit(int x) {
-	(void) unlink(_PATH_CRON_PID);
-	_exit(0);
+static void sigintterm_handler(int x) {
+	got_sigintterm = 1;
 }
 
 static void sigchld_reaper(void) {
@@ -614,15 +680,15 @@ static void sigchld_reaper(void) {
 		case -1:
 			if (errno == EINTR)
 				continue;
-			Debug(DPROC, ("[%ld] sigchld...no children\n", (long) getpid()))
+			Debug(DPROC, ("[%ld] sigchld...no children\n", (long) getpid()));
 				break;
 		case 0:
-			Debug(DPROC, ("[%ld] sigchld...no dead kids\n", (long) getpid()))
+			Debug(DPROC, ("[%ld] sigchld...no dead kids\n", (long) getpid()));
 				break;
 		default:
 			Debug(DPROC,
 				("[%ld] sigchld...pid #%ld died, stat=%d\n",
-					(long) getpid(), (long) pid, WEXITSTATUS(waiter)))
+					(long) getpid(), (long) pid, WEXITSTATUS(waiter)));
 				break;
 		}
 	} while (pid > 0);
@@ -631,7 +697,7 @@ static void sigchld_reaper(void) {
 static void parse_args(int argc, char *argv[]) {
 	int argch;
 
-	while (-1 != (argch = getopt(argc, argv, "hnpsix:m:c"))) {
+	while (-1 != (argch = getopt(argc, argv, "hnpsiPx:m:c"))) {
 		switch (argch) {
 			case 'x':
 				if (!set_debug_flags(optarg))
@@ -649,6 +715,9 @@ static void parse_args(int argc, char *argv[]) {
 			case 'i':
 				DisableInotify = 1;
 				break;
+			case 'P':
+				ChangePath = 0;
+				break;
 			case 'm':
 				strncpy(MailCmd, optarg, MAX_COMMAND);
 				break;
