@@ -27,7 +27,25 @@
  * to add clustering support.
  */
 
-#include <cron.h>
+#include "config.h"
+
+#include <dirent.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <pwd.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <unistd.h>
+
+#ifdef WITH_INOTIFY
+# include <sys/inotify.h>
+#endif
+
+#include "funcs.h"
+#include "globals.h"
+#include "pathnames.h"
 
 #define TMAX(a,b) ((a)>(b)?(a):(b))
 
@@ -45,11 +63,11 @@ static void process_crontab(const char *, const char *,
 static int not_a_crontab(DIR_T * dp);
 /* return 1 if we should skip this file */
 
-static void max_mtime(char *dir_name, struct stat *max_st);
+static void max_mtime(const char *dir_name, struct stat *max_st);
 /* record max mtime of any file under dir_name in max_st */
 
 static int
-check_open(const char *tabname, const char *fname, const char *uname,
+check_open(const char *tabname, const char *uname,
 	struct passwd *pw, time_t * mtime) {
 	struct stat statbuf;
 	int crontab_fd;
@@ -185,10 +203,10 @@ process_crontab(const char *uname, const char *fname, const char *tabname,
 		goto next_crontab;
 	}
 
-	if ((crontab_fd = check_open(tabname, fname, uname, pw, &mtime)) == -1)
+	if ((crontab_fd = check_open(tabname, uname, pw, &mtime)) == -1)
 		goto next_crontab;
 
-	Debug(DLOAD, ("\t%s:", fname))
+	Debug(DLOAD, ("\t%s:", fname));
 
 	if (old_db != NULL)
 		u = find_user(old_db, fname, crond_crontab ? tabname : NULL);	/* find user in old_db */
@@ -198,7 +216,7 @@ process_crontab(const char *uname, const char *fname, const char *tabname,
 		 * in, then we can just use our existing entry.
 		 */
 		if (u->mtime == mtime) {
-			Debug(DLOAD, (" [no change, using old data]"))
+			Debug(DLOAD, (" [no change, using old data]"));
 				unlink_user(old_db, u);
 			link_user(new_db, u);
 			goto next_crontab;
@@ -211,7 +229,7 @@ process_crontab(const char *uname, const char *fname, const char *tabname,
 		 * users will be deleted from the old database when
 		 * we finish with the crontab...
 		 */
-		Debug(DLOAD, (" [delete old data]"))
+		Debug(DLOAD, (" [delete old data]"));
 			unlink_user(old_db, u);
 		free_user(u);
 		log_it(fname, getpid(), "RELOAD", tabname, 0);
@@ -226,7 +244,7 @@ process_crontab(const char *uname, const char *fname, const char *tabname,
 
   next_crontab:
 	if (crontab_fd != -1) {
-		Debug(DLOAD, (" [done]\n"))
+		Debug(DLOAD, (" [done]\n"));
 			close(crontab_fd);
 	}
 }
@@ -281,7 +299,7 @@ void check_inotify_database(cron_db * old_db) {
 	DIR *dir;
 	struct timeval time;
 	fd_set rfds;
-	int retval = 0;
+	int retval;
 	char buf[BUF_LEN];
 	pid_t pid = getpid();
 	time.tv_sec = 0;
@@ -309,7 +327,7 @@ void check_inotify_database(cron_db * old_db) {
 
 		if (retval <= 0) {
 			log_it("CRON", pid, "INOTIFY", "read failed", errno);
-			/* something fatal must have occured we have no other reasonable
+			/* something fatal must have occurred we have no other reasonable
 			 * way how to handle this failure than exit.
 			 */
 			(void) exit(ERROR_EXIT);
@@ -321,8 +339,9 @@ void check_inotify_database(cron_db * old_db) {
 		set_cron_watched(old_db->ifd);
 
 		/* TODO: parse the events and read only affected files */
-
+#if defined ENABLE_SYSCRONTAB
 		process_crontab("root", NULL, SYSCRONTAB, &new_db, old_db);
+#endif
 
 		if (!(dir = opendir(SYS_CROND_DIR))) {
 			log_it("CRON", pid, "OPENDIR FAILED", SYS_CROND_DIR, errno);
@@ -374,7 +393,7 @@ void check_inotify_database(cron_db * old_db) {
 	}
 
 	overwrite_database(old_db, &new_db);
-	Debug(DLOAD, ("check_inotify_database is done\n"))
+	Debug(DLOAD, ("check_inotify_database is done\n"));
 }
 #endif
 
@@ -382,9 +401,9 @@ static void overwrite_database(cron_db * old_db, cron_db * new_db) {
 	user *u, *nu;
 	/* whatever's left in the old database is now junk.
 	 */
-	Debug(DLOAD, ("unlinking old database:\n"))
+	Debug(DLOAD, ("unlinking old database:\n"));
 		for (u = old_db->head; u != NULL; u = nu) {
-		Debug(DLOAD, ("\t%s\n", u->name))
+		Debug(DLOAD, ("\t%s\n", u->name));
 			nu = u->next;
 		unlink_user(old_db, u);
 		free_user(u);
@@ -403,7 +422,7 @@ int load_database(cron_db * old_db) {
 	pid_t pid = getpid();
 	int is_local = 0;
 
-	Debug(DLOAD, ("[%ld] load_database()\n", (long) pid))
+	Debug(DLOAD, ("[%ld] load_database()\n", (long) pid));
 
 		/* before we start loading any data, do a stat on SPOOL_DIR
 		 * so that if anything changes as of this moment (i.e., before we've
@@ -429,10 +448,12 @@ int load_database(cron_db * old_db) {
 		max_mtime(SYS_CROND_DIR, &crond_stat);
 	}
 
+#if defined ENABLE_SYSCRONTAB
 	/* track system crontab file
 	 */
 	if (stat(SYSCRONTAB, &syscron_stat) < OK)
 		syscron_stat.st_mtime = 0;
+#endif
 
 	/* if spooldir's mtime has not changed, we don't need to fiddle with
 	 * the database.
@@ -445,7 +466,7 @@ int load_database(cron_db * old_db) {
 			TMAX(statbuf.st_mtime, syscron_stat.st_mtime))
 		) {
 		Debug(DLOAD, ("[%ld] spool dir mtime unch, no load needed.\n",
-				(long) pid))
+				(long) pid));
 			return 0;
 	}
 
@@ -461,8 +482,10 @@ int load_database(cron_db * old_db) {
 	new_db.ifd = old_db->ifd;
 #endif
 
+#if defined ENABLE_SYSCRONTAB
 	if (syscron_stat.st_mtime)
 		process_crontab("root", NULL, SYSCRONTAB, &new_db, old_db);
+#endif
 
 	if (!(dir = opendir(SYS_CROND_DIR))) {
 		log_it("CRON", pid, "OPENDIR FAILED", SYS_CROND_DIR, errno);
@@ -518,7 +541,7 @@ int load_database(cron_db * old_db) {
 	endpwent();
 
 	overwrite_database(old_db, &new_db);
-	Debug(DLOAD, ("load_database is done\n"))
+	Debug(DLOAD, ("load_database is done\n"));
 		return 1;
 }
 
@@ -594,7 +617,7 @@ static int not_a_crontab(DIR_T * dp) {
 	return (0);
 }
 
-static void max_mtime(char *dir_name, struct stat *max_st) {
+static void max_mtime(const char *dir_name, struct stat *max_st) {
 	DIR *dir;
 	DIR_T *dp;
 	struct stat st;
