@@ -19,9 +19,28 @@
  * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-#include <cron.h>
+#include "config.h"
 
-static int child_process(entry *, user *, char **);
+#include <ctype.h>
+#include <errno.h>
+#include <pwd.h>
+#include <signal.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/wait.h>
+#include <unistd.h>
+
+#include "externs.h"
+#include "funcs.h"
+#include "globals.h"
+#include "structs.h"
+
+#ifndef isascii
+# define isascii(c)	((unsigned)(c)<=0177)
+#endif
+
+static int child_process(entry *, char **);
 static int safe_p(const char *, const char *);
 
 void do_command(entry * e, user * u) {
@@ -31,7 +50,7 @@ void do_command(entry * e, user * u) {
 
 	Debug(DPROC, ("[%ld] do_command(%s, (%s,%ld,%ld))\n",
 			(long) pid, e->cmd, u->name,
-			(long) e->pwd->pw_uid, (long) e->pwd->pw_gid))
+			(long) e->pwd->pw_uid, (long) e->pwd->pw_gid));
 
 		/* fork to become asynchronous -- parent process is done immediately,
 		 * and continues to run the normal cron code, which means return to
@@ -52,20 +71,22 @@ void do_command(entry * e, user * u) {
 		if (cron_set_job_security_context(e, u, &jobenv) != 0) {
 			_exit(ERROR_EXIT);
 		}
-		ev = child_process(e, u, jobenv);
+		ev = child_process(e, jobenv);
+#ifdef WITH_PAM
 		cron_close_pam();
+#endif
 		env_free(jobenv);
-		Debug(DPROC, ("[%ld] child process done, exiting\n", (long) getpid()))
+		Debug(DPROC, ("[%ld] child process done, exiting\n", (long) getpid()));
 		_exit(ev);
 		break;
 	default:
 		/* parent process */
 		break;
 	}
-	Debug(DPROC, ("[%ld] main process returning to work\n", (long) pid))
+	Debug(DPROC, ("[%ld] main process returning to work\n", (long) pid));
 }
 
-static int child_process(entry * e, user * u, char **jobenv) {
+static int child_process(entry * e, char **jobenv) {
 	int stdin_pipe[2], stdout_pipe[2];
 	char *input_data, *usernm, *mailto, *mailfrom;
 	int children = 0;
@@ -87,7 +108,7 @@ static int child_process(entry * e, user * u, char **jobenv) {
 	sigaction(SIGCHLD, &sa, NULL);
 
 
-	Debug(DPROC, ("[%ld] child_process('%s')\n", (long) getpid(), e->cmd))
+	Debug(DPROC, ("[%ld] child_process('%s')\n", (long) getpid(), e->cmd));
 #ifdef CAPITALIZE_FOR_PS
 		/* mark ourselves as different to PS command watchers by upshifting
 		 * our program name.  This has no effect on some kernels.
@@ -163,7 +184,7 @@ static int child_process(entry * e, user * u, char **jobenv) {
 		return ERROR_EXIT;
 		/*NOTREACHED*/
 	case 0:
-		Debug(DPROC, ("[%ld] grandchild process fork()'ed\n", (long) getpid()))
+		Debug(DPROC, ("[%ld] grandchild process fork()'ed\n", (long) getpid()));
 
 		/* write a log message.  we've waited this long to do it
 		 * because it was not until now that we knew the PID that
@@ -217,6 +238,12 @@ static int child_process(entry * e, user * u, char **jobenv) {
 		 */
 		{
 			char *shell = env_get("SHELL", jobenv);
+			int fd, fdmax = getdtablesize();
+
+			/* close all unwanted open file descriptors */
+			for(fd = STDERR + 1; fd < fdmax; fd++) {
+				close(fd);
+			}
 
 #if DEBUGGING
 			if (DebugFlags & DTEST) {
@@ -243,7 +270,7 @@ static int child_process(entry * e, user * u, char **jobenv) {
 	 * the user's command.
 	 */
 
-	Debug(DPROC, ("[%ld] child continues, closing pipes\n", (long) getpid()))
+	Debug(DPROC, ("[%ld] child continues, closing pipes\n", (long) getpid()));
 
 	/* close the ends of the pipe that will only be referenced in the
 	 * grandchild process...
@@ -269,7 +296,7 @@ static int child_process(entry * e, user * u, char **jobenv) {
 		int ch;
 
 		Debug(DPROC, ("[%ld] child2 sending data to grandchild\n",
-				(long) getpid()))
+				(long) getpid()));
 
 		/* reset the SIGPIPE back to default so the child will terminate
 		 * if it tries to write to a closed pipe
@@ -315,7 +342,7 @@ static int child_process(entry * e, user * u, char **jobenv) {
 		fclose(out);
 
 		Debug(DPROC, ("[%ld] child2 done sending to grandchild\n",
-				(long) getpid()))
+				(long) getpid()));
 		_exit(0);
 	}
 
@@ -334,7 +361,7 @@ static int child_process(entry * e, user * u, char **jobenv) {
 	 */
 
 	Debug(DPROC, ("[%ld] child reading output from grandchild\n",
-			(long) getpid()))
+			(long) getpid()));
 
 	/*local */  {
 		FILE *in = fdopen(stdout_pipe[READ_PIPE], "r");
@@ -355,7 +382,7 @@ static int child_process(entry * e, user * u, char **jobenv) {
 
 			Debug(DPROC | DEXT,
 				("[%ld] got data (%x:%c) from grandchild\n",
-					(long) getpid(), ch, ch))
+					(long) getpid(), ch, ch));
 
 				/* get name of recipient.  this is MAILTO if set to a
 				 * valid local username; USER otherwise.
@@ -389,7 +416,7 @@ static int child_process(entry * e, user * u, char **jobenv) {
 
 			/* Also skip it if MailCmd is set to "off" */
 			if (mailto && safe_p(usernm, mailto)
-			           && strncmp(MailCmd,"off",4)) {
+				&& strncmp(MailCmd,"off",3) && !SyslogOutput) {
 				char **env;
 				char mailcmd[MAX_COMMAND];
 				char hostname[MAXHOSTNAMELEN];
@@ -414,7 +441,7 @@ static int child_process(entry * e, user * u, char **jobenv) {
 					(void) _exit(ERROR_EXIT);
 				}
 
-				fprintf(mail, "From: %s (Cron Daemon)\n", mailfrom);
+				fprintf(mail, "From: \"(Cron Daemon)\" <%s>\n", mailfrom);
 				fprintf(mail, "To: %s\n", mailto);
 				fprintf(mail, "Subject: Cron <%s@%s> %s\n",
 					usernm, first_word(hostname, "."), e->cmd);
@@ -455,6 +482,7 @@ static int child_process(entry * e, user * u, char **jobenv) {
 				 * defined (and suggested by) RFC3834.
 				 */
 				fprintf(mail, "Auto-Submitted: auto-generated\n");
+				fprintf(mail, "Precedence: bulk\n");
 
 				for (env = jobenv; *env; env++)
 					fprintf(mail, "X-Cron-Env: <%s>\n", *env);
@@ -493,7 +521,7 @@ static int child_process(entry * e, user * u, char **jobenv) {
 			 */
 
 			if (mail) {
-				Debug(DPROC, ("[%ld] closing pipe to mail\n", (long) getpid()))
+				Debug(DPROC, ("[%ld] closing pipe to mail\n", (long) getpid()));
 					/* Note: the pclose will probably see
 					 * the termination of the grandchild
 					 * in addition to the mail process, since
@@ -515,7 +543,7 @@ static int child_process(entry * e, user * u, char **jobenv) {
 			 * log the facts so the poor user can figure out
 			 * what's going on.
 			 */
-			if (mail && status) {
+			if (mail && status && !SyslogOutput) {
 				char buf[MAX_TEMPSTR];
 
 				sprintf(buf,
@@ -526,7 +554,7 @@ static int child_process(entry * e, user * u, char **jobenv) {
 
 		}	/*if data from grandchild */
 
-		Debug(DPROC, ("[%ld] got EOF from grandchild\n", (long) getpid()))
+		Debug(DPROC, ("[%ld] got EOF from grandchild\n", (long) getpid()));
 
 		fclose(in);	/* also closes stdout_pipe[READ_PIPE] */
 	}
@@ -535,22 +563,22 @@ static int child_process(entry * e, user * u, char **jobenv) {
 	 */
 	for (; children > 0; children--) {
 		WAIT_T waiter;
-		PID_T pid;
+		PID_T child;
 
 		Debug(DPROC, ("[%ld] waiting for grandchild #%d to finish\n",
-				(long) getpid(), children))
-			while ((pid = wait(&waiter)) < OK && errno == EINTR) ;
-		if (pid < OK) {
+				(long) getpid(), children));
+			while ((child = wait(&waiter)) < OK && errno == EINTR) ;
+		if (child < OK) {
 			Debug(DPROC,
 				("[%ld] no more grandchildren--mail written?\n",
-					(long) getpid()))
+					(long) getpid()));
 				break;
 		}
 		Debug(DPROC, ("[%ld] grandchild #%ld finished, status=%04x",
-				(long) getpid(), (long) pid, WEXITSTATUS(waiter)))
+				(long) getpid(), (long) child, WEXITSTATUS(waiter)));
 			if (WIFSIGNALED(waiter) && WCOREDUMP(waiter))
-			Debug(DPROC, (", dumped core"))
-				Debug(DPROC, ("\n"))
+			Debug(DPROC, (", dumped core"));
+				Debug(DPROC, ("\n"));
 	}
 	return OK_EXIT;
 }
