@@ -19,7 +19,20 @@
  * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-#include <cron.h>
+#include "config.h"
+
+#include <ctype.h>
+#include <errno.h>
+#include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
+
+#include "globals.h"
+#include "funcs.h"
+
+#if defined(BSD)
+extern char **environ;
+#endif
 
 char **env_init(void) {
 	char **p = (char **) malloc(sizeof (char *));
@@ -59,7 +72,7 @@ char **env_copy(char **envp) {
 	return (p);
 }
 
-char **env_set(char **envp, char *envstr) {
+char **env_set(char **envp, const char *envstr) {
 	int count, found;
 	char **p, *envtmp;
 
@@ -104,6 +117,48 @@ char **env_set(char **envp, char *envstr) {
 	return (p);
 }
 
+int env_set_from_environ(char ***envpp) {
+	static const char *names[] = {
+		"LANG",
+		"LC_CTYPE",
+		"LC_NUMERIC",
+		"LC_TIME",
+		"LC_COLLATE",
+		"LC_MONETARY",
+		"LC_MESSAGES",
+		"LC_PAPER",
+		"LC_NAME",
+		"LC_ADDRESS",
+		"LC_TELEPHONE",
+		"LC_MEASUREMENT",
+		"LC_IDENTIFICATION",
+		"LC_ALL",
+		"LANGUAGE",
+		"RANDOM_DELAY",
+		NULL
+	};
+	const char **name;
+	char **procenv;
+
+	for (procenv = environ; *procenv != NULL; ++procenv) {
+		for (name = names; *name != NULL; ++name) {
+			size_t namelen;
+
+			namelen = strlen(*name);
+			if (strncmp(*name, *procenv, namelen) == 0 
+			    && (*procenv)[namelen] == '=') {
+				char **tmpenv;
+
+				tmpenv = env_set(*envpp, *procenv);
+				if (tmpenv == NULL)
+					return FALSE;
+				*envpp = tmpenv;
+			}
+		}
+	}
+	return TRUE;
+}
+
 /* The following states are used by load_env(), traversed in order: */
 enum env_state {
 	NAMEI,	/* First char of NAME, may be quote */
@@ -133,10 +188,10 @@ int load_env(char *envstr, FILE * f) {
 	if (EOF == get_string(envstr, MAX_ENVSTR, f, "\n"))
 		return (ERR);
 
-	Debug(DPARS, ("load_env, read <%s>\n", envstr))
+	Debug(DPARS, ("load_env, read <%s>\n", envstr));
 
-		bzero(name, sizeof name);
-	bzero(val, sizeof val);
+		memset(name, 0, sizeof name);
+	memset(val, 0, sizeof val);
 	str = name;
 	state = NAMEI;
 	quotechar = '\0';
@@ -204,7 +259,7 @@ int load_env(char *envstr, FILE * f) {
 		}
 	}
 	if (state != FINI && !(state == VALUE && !quotechar)) {
-		Debug(DPARS, ("load_env, not an env var, state = %d\n", state))
+		Debug(DPARS, ("load_env, not an env var, state = %d\n", state));
 			fseek(f, filepos, 0);
 		Set_LineNum(fileline);
 		return (FALSE);
@@ -224,11 +279,11 @@ int load_env(char *envstr, FILE * f) {
 	 */
 	if (!glue_strings(envstr, MAX_ENVSTR, name, val, '='))
 		return (FALSE);
-	Debug(DPARS, ("load_env, <%s> <%s> -> <%s>\n", name, val, envstr))
+	Debug(DPARS, ("load_env, <%s> <%s> -> <%s>\n", name, val, envstr));
 		return (TRUE);
 }
 
-char *env_get(char *name, char **envp) {
+char *env_get(const char *name, char **envp) {
 	int len = strlen(name);
 	char *p, *q;
 
