@@ -23,7 +23,23 @@
  * vix 30dec86 [written]
  */
 
-#include <cron.h>
+#include "config.h"
+
+#include "globals.h"
+
+#include <ctype.h>
+#include <errno.h>
+#include <signal.h>
+#include <stdarg.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/stat.h>
+
+#if defined(SYSLOG)
+# include <syslog.h>
+#endif
+
 #ifdef WITH_AUDIT
 # include <libaudit.h>
 #endif
@@ -37,7 +53,10 @@
 #ifdef HAVE_FLOCK	/* flock(2) */
 # include <sys/file.h>
 #endif
-#include <stdio.h>
+
+#include "funcs.h"
+#include "macros.h"
+#include "pathnames.h"
 
 #if defined(SYSLOG) && defined(LOG_FILE)
 # undef LOG_FILE
@@ -57,7 +76,9 @@ static int LogFD = ERR;
 static int syslog_open = FALSE;
 #endif
 
-#if defined(HAVE_FCNTL) && defined(F_SETLK)
+#if defined(HAVE_FLOCK)
+# define trylock_file(fd)      flock((fd), LOCK_EX|LOCK_NB)
+#elif defined(HAVE_FCNTL) && defined(F_SETLK)
 static int trylock_file(int fd) {
 	struct flock fl;
 
@@ -71,8 +92,6 @@ static int trylock_file(int fd) {
 }
 #elif defined(HAVE_LOCKF)
 # define trylock_file(fd)      lockf((fd), F_TLOCK, 0)
-#elif defined(HAVE_FLOCK)
-# define trylock_file(fd)      flock((fd), LOCK_EX|LOCK_NB)
 #endif
 
 /*
@@ -290,6 +309,9 @@ void acquire_daemonlock(int closeflag) {
 			close(fd);
 			fd = -1;
 		}
+		/* and restore default sig handlers so we don't remove pid file if killed */
+		signal(SIGINT,SIG_DFL);
+		signal(SIGTERM,SIG_DFL);
 		return;
 	}
 
@@ -308,7 +330,7 @@ void acquire_daemonlock(int closeflag) {
 		if (trylock_file(fd) < OK) {
 			int save_errno = errno;
 
-			bzero(buf, sizeof (buf));
+			memset(buf, 0, sizeof (buf));
 			if ((num = read(fd, buf, sizeof (buf) - 1)) > 0 &&
 				(otherpid = strtol(buf, &ep, 10)) > 0 &&
 				ep != buf && *ep == '\n' && otherpid != LONG_MAX) {
@@ -327,6 +349,13 @@ void acquire_daemonlock(int closeflag) {
 		(void) fchmod(fd, 0644);
 		(void) fcntl(fd, F_SETFD, 1);
 	}
+#if !defined(HAVE_FLOCK)
+	else {
+		/* Racy but better than nothing, just hope the parent exits */
+		sleep(0);
+		trylock_file(fd);	
+	}
+#endif
 
 	sprintf(buf, "%ld\n", (long) pid);
 	(void) lseek(fd, (off_t) 0, SEEK_SET);
@@ -368,7 +397,7 @@ void unget_char(int ch, FILE * file) {
  *      (3) uses get_char() so LineNumber will be accurate
  *      (4) returns EOF or terminating character, whichever
  */
-int get_string(char *string, int size, FILE * file, char *terms) {
+int get_string(char *string, int size, FILE * file, const char *terms) {
 	int ch;
 
 	while (EOF != (ch = get_char(file)) && !strchr(terms, ch)) {
@@ -582,7 +611,7 @@ void log_close(void) {
 #endif	 /*SYSLOG*/
 }
 
-/* char *first_word(char *s, char *t)
+/* char *first_word(const char *s, const char *t)
  *	return pointer to first word
  * parameters:
  *	s - string we want the first word of
@@ -591,7 +620,7 @@ void log_close(void) {
  *	(1) this routine is fairly slow
  *	(2) it returns a pointer to static storage
  */
-char *first_word(char *s, char *t) {
+char *first_word(const char *s, const char *t) {
 	static char retbuf[2][MAX_TEMPSTR + 1];	/* sure wish C had GC */
 	static int retsel = 0;
 	char *rb, *rp;
@@ -619,7 +648,7 @@ char *first_word(char *s, char *t) {
 /* warning:
  *	heavily ascii-dependent.
  */
-void mkprint(char *dst, unsigned char *src, int len) {
+static void mkprint(char *dst, unsigned char *src, int len) {
 /*
  * XXX
  * We know this routine can't overflow the dst buffer because mkprints()
