@@ -30,11 +30,24 @@
  *
  */
 
+#include "config.h"
+
+#include <errno.h>
+#include <pwd.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/wait.h>
+#include <unistd.h>
+
+#include "funcs.h"
+#include "globals.h"
+#include "macros.h"
+
 #ifdef HAVE_SYS_CDEFS_H
 # include <sys/cdefs.h>
 #endif
 
-#include <cron.h>
 #include <signal.h>
 
 /*
@@ -56,6 +69,7 @@ FILE *cron_popen(char *program, const char *type, struct passwd *pw) {
 	ssize_t out;
 	char buf[PIPE_BUF];
 	struct sigaction sa;
+	int fd;
 
 #ifdef __GNUC__
 	(void) &iop;	/* Avoid fork clobbering */
@@ -69,7 +83,7 @@ FILE *cron_popen(char *program, const char *type, struct passwd *pw) {
 			return (NULL);
 		if (!(pids = (PID_T *) malloc((u_int) (fds * sizeof (PID_T)))))
 			return (NULL);
-		bzero((char *) pids, fds * sizeof (PID_T));
+		memset((char *) pids, 0, fds * sizeof (PID_T));
 	}
 	if (pipe(pdes) < 0)
 		return (NULL);
@@ -108,6 +122,11 @@ FILE *cron_popen(char *program, const char *type, struct passwd *pw) {
 		sa.sa_handler = SIG_DFL;
 		sigaction(SIGPIPE, &sa, NULL);
 
+		/* close all unwanted open file descriptors */
+		for (fd = STDERR + 1; fd < fds; fd++) {
+			close(fd);
+		}
+
 		if (cron_change_user_permanently(pw, pw->pw_dir) != 0)
 			_exit(2);
 
