@@ -19,7 +19,24 @@
  * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-#include <cron.h>
+#include "config.h"
+
+#include <errno.h>
+#include <grp.h>
+#include <pwd.h>
+#include <sys/types.h>
+#include <unistd.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "cronie_common.h"
+#include "funcs.h"
+#include "globals.h"
+#include "macros.h"
+
+#ifdef WITH_PAM
+# include <security/pam_appl.h>
+#endif
 
 #ifdef WITH_SELINUX
 # include <selinux/selinux.h>
@@ -39,7 +56,8 @@ static int pam_session_opened = 0;	//global for open session
 
 static int
 cron_conv(int num_msg, const struct pam_message **msgm,
-        struct pam_response **response, void *appdata_ptr)
+	struct pam_response **response ATTRIBUTE_UNUSED,
+	void *appdata_ptr ATTRIBUTE_UNUSED)
 {
         int i;
 
@@ -82,13 +100,14 @@ static int cron_get_job_range(user * u, security_context_t * ucontextp,
 	char **jobenv);
 #endif
 
-void cron_restore_default_security_context() {
+void cron_restore_default_security_context(void) {
 #ifdef WITH_SELINUX
 	setexeccon(NULL);
 #endif
 }
 
-int cron_set_job_security_context(entry * e, user * u, char ***jobenv) {
+int cron_set_job_security_context(entry *e, user *u ATTRIBUTE_UNUSED,
+	char ***jobenv) {
 	time_t minutely_time = 0;
 #ifdef WITH_PAM
 	int ret;
@@ -99,7 +118,7 @@ int cron_set_job_security_context(entry * e, user * u, char ***jobenv) {
 		 * Ensure that these jobs never run in the same minute:
 		 */
 		minutely_time = time(0);
-		Debug(DSCH, ("Minute-ly job. Recording time %lu\n", minutely_time))
+		Debug(DSCH, ("Minute-ly job. Recording time %lu\n", minutely_time));
 	}
 
 #ifdef WITH_PAM
@@ -110,15 +129,13 @@ int cron_set_job_security_context(entry * e, user * u, char ***jobenv) {
 	}
 #endif
 
-	*jobenv = build_env(e->envp);
-
 #ifdef WITH_SELINUX
 	/* we must get the crontab context BEFORE changing user, else
 	 * we'll not be permitted to read the cron spool directory :-)
 	 */
 	security_context_t ucontext = 0;
 
-	if (cron_get_job_range(u, &ucontext, *jobenv) < OK) {
+	if (cron_get_job_range(u, &ucontext, e->envp) < OK) {
 		log_it(e->pwd->pw_name, getpid(), "ERROR",
 			"failed to get SELinux context", 0);
 		return -1;
@@ -146,6 +163,8 @@ int cron_set_job_security_context(entry * e, user * u, char ***jobenv) {
 		return -1;
 	}
 
+	*jobenv = build_env(e->envp);
+
 	time_t job_run_time = time(0L);
 
 	if ((minutely_time > 0) && ((job_run_time / 60) != (minutely_time / 60))) {
@@ -185,18 +204,18 @@ int cron_start_pam(struct passwd *pw) {
 	return retcode;
 }
 
-static int cron_open_pam_session(struct passwd *pw) {
-	int retcode = 0;
-
 #if defined(WITH_PAM)
+static int cron_open_pam_session(struct passwd *pw) {
+	int retcode;
+
 	retcode = pam_open_session(pamh, PAM_SILENT);
 	PAM_FAIL_CHECK;
 	if (retcode == PAM_SUCCESS)
 		pam_session_opened = 1;
-#endif
 
 	return retcode;
 }
+#endif
 
 void cron_close_pam(void) {
 #if defined(WITH_PAM)
@@ -247,9 +266,9 @@ int cron_change_user_permanently(struct passwd *pw, char *homedir) {
 }
 
 
+#ifdef WITH_SELINUX
 static int cron_authorize_context(security_context_t scontext,
 	security_context_t file_context) {
-#ifdef WITH_SELINUX
 	struct av_decision avd;
 	int retval;
 	security_class_t tclass;
@@ -276,13 +295,13 @@ static int cron_authorize_context(security_context_t scontext,
 		tclass, bit, &avd);
 	if (retval || ((bit & avd.allowed) != bit))
 		return 0;
-#endif
 	return 1;
 }
+#endif
 
+#ifdef WITH_SELINUX
 static int cron_authorize_range(security_context_t scontext,
 	security_context_t ucontext) {
-#ifdef WITH_SELINUX
 	struct av_decision avd;
 	int retval;
 	security_class_t tclass;
@@ -309,9 +328,9 @@ static int cron_authorize_range(security_context_t scontext,
 
 	if (retval || ((bit & avd.allowed) != bit))
 		return 0;
-#endif
 	return 1;
 }
+#endif
 
 #if WITH_SELINUX
 /* always uses u->scontext as the default process context, then changes the
@@ -424,7 +443,7 @@ static int cron_change_selinux_range(user * u, security_context_t ucontext) {
 		}
 	}
 
-	if (setexeccon(ucontext) < 0 || setkeycreatecon(ucontext) < 0) {
+	if (setexeccon(ucontext) < 0) {
 		if (security_getenforce() > 0) {
 			if (asprintf
 				(&msg, "Could not set exec or keycreate context to %s for user",
@@ -437,7 +456,7 @@ static int cron_change_selinux_range(user * u, security_context_t ucontext) {
 		else {
 			if (asprintf
 				(&msg,
-					"Could not set exec or keycreate context to %s for user,"
+					"Could not set exec context to %s for user,"
 					" but SELinux in permissive mode, continuing",
 					(char *) ucontext) >= 0) {
 				log_it(u->name, getpid(), "WARNING", msg, 0);
@@ -450,12 +469,13 @@ static int cron_change_selinux_range(user * u, security_context_t ucontext) {
 }
 #endif
 
+#ifdef WITH_SELINUX
 int
 get_security_context(const char *name, int crontab_fd,
 	security_context_t * rcontext, const char *tabname) {
-#ifdef WITH_SELINUX
 	security_context_t scontext = NULL;
 	security_context_t file_context = NULL;
+	security_context_t rawcontext=NULL;
 	int retval = 0;
 	char *seuser = NULL;
 	char *level = NULL;
@@ -474,6 +494,10 @@ get_security_context(const char *name, int crontab_fd,
 
 	retval = get_default_context_with_level(name == NULL ? "system_u" : seuser,
 		level, NULL, &scontext);
+	if (selinux_trans_to_raw_context(scontext, &rawcontext) == 0) {
+		freecon(scontext);
+		scontext = rawcontext;
+	}
 	free(seuser);
 	free(level);
 	if (retval) {
@@ -528,21 +552,21 @@ get_security_context(const char *name, int crontab_fd,
 	freecon(file_context);
 
 	*rcontext = scontext;
-#endif
 	return 0;
 }
+#endif
 
-void free_security_context(security_context_t * scontext) {
 #ifdef WITH_SELINUX
+void free_security_context(security_context_t * scontext) {
 	if (*scontext != NULL) {
 		freecon(*scontext);
 		*scontext = 0L;
 	}
-#endif
 }
+#endif
 
-int crontab_security_access(void) {
 #ifdef WITH_SELINUX
+int crontab_security_access(void) {
 	int selinux_check_passwd_access = -1;
 	if (is_selinux_enabled() > 0) {
 		security_context_t user_context;
@@ -582,20 +606,27 @@ int crontab_security_access(void) {
 
 		return selinux_check_passwd_access;
 	}
-#endif
 	return 0;
 }
+#endif
 
 /* Build up the job environment from the PAM environment plus the
 * crontab environment 
 */
 static char **build_env(char **cronenv) {
 #ifdef WITH_PAM
-	char **jobenv = cronenv;
-	char **pamenv = pam_getenvlist(pamh);
+	char **jobenv = pam_getenvlist(pamh);
 	char *cronvar;
 	int count = 0;
-	jobenv = env_copy(pamenv);
+
+	if (jobenv == NULL) {
+		jobenv = env_init();
+		if (jobenv == NULL) {
+			log_it("CRON", getpid(),
+				"ERROR", "Initialization of cron environment variables failed", 0);
+			return NULL;
+		}
+	}
 
 	/* Now add the cron environment variables. Since env_set()
 	 * overwrites existing variables, this will let cron's
